name: "nergi_segmentation"
revision: 10
description: "segmentation: resolution 0.25; SVM segmentation (svm_segm_20160905); cut convexities for multiple fruits separation (version Aliquis 1.16.5); shape polygon"

stages {
  name: "src"
  type: SOURCE_CUSTOM
#  custom_param {
#    proxy: "test:ConcatCustomSource"
#    args: "source_test.apl"
#  }
}

# Reduce image resolution for computation al issues
stages {
	name: "src_resize"
	type: RESIZE
	input: "src"
	resize_param {
		fx: 0.25
		fy: 0.25
	}
}

# Perform segmentation with a SVM
stages {
	name: "fg_areas"
	type: SVM_SEGMENTATION
	input: "src_resize"
	svm_segmentation_param {
		model_file: "models/svm_segm_20160905/model"
		scale: 0.0039215686
	}
}

# Morphological opening to remove the residual segmentation errors
stages {
	name: "morph"
	type: MORPHOLOGY
	input: "fg_areas"
	morphology_param {
		type: OPEN
		iterations_num: 2
		kernel {
			shape_str: "5x5"
			d:0   d:0   d:255 d:0   d:0
			d:0   d:255 d:255 d:255 d:0
			d:255 d:255 d:255 d:255 d:255
			d:0   d:255 d:255 d:255 d:0
			d:0   d:0   d:255 d:0   d:0
		}
	}
}

# Mask resize to return to the original image size
stages {
	name: "bitmask_resize"
	type: RESIZE
	input: "morph"
	resize_param {
		fx: 4
		fy: 4
	}
}

# Cut convexities for multiple fruits separation for Aliquis 1.16.6
stages {
	name: "cut_convexity"
	type: CUT_MASK_CONVEXITIES
	input: "bitmask_resize"
	cut_mask_convexities_param {
		type: SINGLE
		depth_min: 100
		distance_max: 250
		x_diff_max: 30
		vertical: true
		horizontal: false
		line_thickness: 1
		iterations_num: 5
	}
}

# Cut convexities for multiple fruits separation for Aliquis 1.16.5
#stages {
#	name: "cut_convexity"
#	type: CUT_MASK_CONVEXITIES
#	input: "bitmask_resize"
#	cut_mask_convexities_param {
#		convexity_depth_min: 100
#		line_thickness: 4
#		iterations_num: 2
#		convexities_distance_max: 250
#		x_diff_max: 30
#	}
#}

# Remove small and/or big residual areas in the mask
stages {
	name: "remove_areas"
	type: REMOVE_AREAS
	input: "cut_convexity"
	remove_areas_param {
		threshold_min: 25000
		#threshold_max: 250000
	}
}

# Find contours of the objects in the bitmask, optionally fit them with some shapes and associates the results with the BGR image
stages {
	name: "item_shapes"
	type: SEGMENTATION
	input: "src"
	input: "remove_areas"
	segmentation_param {
		type: CONTOURS # CONTOURS; DISTANCE_TRANSFORM
		shape: POLYGON # POLYGON (no fitting); INNER_CIRCLE; OUTER_CIRCLE; RECTANGLE; ROTATED_RECTANGLE; ELLIPSE
		convex_hull: false
		match_parents: true
	}
}

# Cut out the bounding-boxes containing the shapes found in the previous stage. The parameter border_policy regards the cases near the image borders
stages {
	name: "item"
	type: CUT_OUT
	input: "item_shapes"
	cut_out_param {
		type: BOUNDING_BOX # BOUNDING_BOX; SQUARED
		border_policy: KEEP # KEEP; DISCARD
		margin_x_rel: 0.2 # = 2*left_margin = 2*right_margin
		margin_y_rel: 0.2 # = 2*top_margin = 2*bottom_margin
	}
}
